%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code is part of the empirical application based on Angrist and Evans
% 1998. The file below allows us to test for whether the individual ATEs
% satisfy certain shape restrictions, and obtain confidence regions for the
% aggregate ATEs while imposing shape restrictions across individual ATEs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set up some basic variables/parameters for the model
SEED = 210;                                 % Set the seed for the random number generator
B = 500;                                    % The number of bootstrap iterations                     
CoverCI = 0.95;                             % Coverage CI
pRn = 0.95;                                 % The quantile of sup norm Rn is chosen from
%Rn = 0;
SubMon = 1;                                 % An indicator for whether we have monotonicity in subgroups (comp, nt, at)
SubPos = 0;                                 % An indicator for whether we have positivity in subgroups (comp, nt, at)

% Set up some parameters for optimization and inversion
linoptions = optimset('linprog');
linoptions.Display = 'off';

zeroptions = optimset('Display','iter'); 
zeroptions.TolX = 1e-5;
zeroptions.TolFun = 1/B;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load the data and keep some of the basic information we want
load ProcAE;

N = size(AllVars,1);           % The sample size
D = AllVars.morekids;          % Treatment: Indicator for more than 2 kids
Y = AllVars.workedm;           % Dependent: Indicator for whether mother worked
Z = AllVars.samesex;           % Instrument: Sex composition of first two kids
edu = AllVars.educm;           % Mother's education

clear AllVars;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Next prepare the variables for our inference procedures
XEdu = [(edu < 9) (edu == 9) (edu == 10) (edu == 11) (edu == 12) (edu == 13) (edu== 14) (edu == 15) (edu == 16) (edu > 16)];

% Declare some helpful dimensions we will use throughout
TotD = size(XEdu,2);            % Number of education dummies
betadim = 8*TotD;               % (2Totd x 2 for cond expects), (2TotD for LATEs), and 2 x NotID
momdim = 6*TotD;                % Number of linear moments

% Set up the variables we will use to build the moments
AllY = [Y, D, (1-D)];                                               % All the Ys we will use
AllX = [XEdu, XEdu.*repmat(D,1,TotD), zeros(N,1)];                  % We add zeros to knock coeffs out when needed
AllZ = [XEdu, XEdu.*repmat(Z,1,TotD), XEdu.*repmat(1-Z,1,TotD), ...
    XEdu.*repmat(D.*(1-Z),1,TotD), XEdu.*repmat((1-D).*Z,1,TotD)];  % All the Zs we'll use

% Setup indicators telling us how moments are formed. The outcome of doing 
% IV/OLS on below should be: P(a), P(n), E[Y|a], E[Y|n], and LATE params
ZeroI = size(AllX,2);
IndY = [2*ones(TotD,1); 3*ones(TotD,1); ones(4*TotD,1)];

IndX = [repmat(1:TotD,TotD,1), ones(TotD,7*TotD)*ZeroI];
IndX = [IndX; ones(TotD,TotD)*ZeroI, repmat(1:TotD,TotD,1), ones(TotD,6*TotD)*ZeroI];
IndX = [IndX; ones(TotD,2*TotD)*ZeroI, repmat(1:TotD,TotD,1), ones(TotD,5*TotD)*ZeroI];
IndX = [IndX; ones(TotD,3*TotD)*ZeroI, repmat(1:TotD,TotD,1), ones(TotD,4*TotD)*ZeroI];
IndX = [IndX; ones(2*TotD,4*TotD)*ZeroI, repmat(1:TotD,2*TotD,1), repmat(TotD+1:2*TotD,2*TotD,1), ones(2*TotD,2*TotD)*ZeroI];

IndZ = [(2*TotD+1:3*TotD)'; (TotD+1:2*TotD)'; (3*TotD+1:5*TotD)'; (1:2*TotD)'];
    
clear D Y Z edu;

% Set up a matrix indicating what education dummies we'll aggregate over
% NOTE: The code here is related to the specification of XEdu earlier
BigATE = [0, ones(1,3), zeros(1,6)];                     % Some HS
BigATE = [BigATE; zeros(1,5), ones(1,3) zeros(1,2)];     % Some Coll 
BigATE = [BigATE; zeros(1,8), ones(1,2)];                % At least Coll
BigATE = [BigATE; ones(1,TotD)];                         % Overall LATE

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set up the matrix of shape constraints. Not this is linked to XEdu
% We start by creating the upper and lower bounds on the missing parameters
% In the partial ID ordering as E[Y(0)|AT] and E[Y(1)|NT]
AiL = [zeros(TotD,6*TotD), -eye(TotD), zeros(TotD,TotD)];
AiL = [AiL; zeros(TotD,6*TotD), zeros(TotD,TotD), -eye(TotD)];
AiU = [zeros(TotD,6*TotD), eye(TotD), zeros(TotD,TotD)];
AiU = [AiU; zeros(TotD,6*TotD), zeros(TotD,TotD), eye(TotD)];
BiL = zeros(2*TotD,1);
BiU = ones(2*TotD,1);
Ai = [AiL; AiU];
Bi = [BiL; BiU];

clear AiL AiU BiL BiU ZeroI;

% Check if we've imposed monotonicity in components
if SubMon == 1
    % Set up monotonicity constraint for compliers
    AiMc = [zeros(TotD-1,5*TotD), eye(TotD-1), zeros(TotD-1,1), zeros(TotD-1,2*TotD)];
    AiMc = AiMc - [zeros(TotD-1,5*TotD), zeros(TotD-1,1), eye(TotD-1), zeros(TotD-1,2*TotD)];
    
    % Set up monotonicity constraint for never takers
    AiMn = [zeros(TotD-1,3*TotD), -eye(TotD-1), zeros(TotD-1,3*TotD+1), eye(TotD-1), zeros(TotD-1,1)];                
    AiMn = AiMn - [zeros(TotD-1,3*TotD+1), -eye(TotD-1), zeros(TotD-1,3*TotD+1), eye(TotD-1)];
                
    % Set up monotonicity constraint for always takers
    AiMa = [zeros(TotD-1,2*TotD), eye(TotD-1), zeros(TotD-1,3*TotD+1), -eye(TotD-1), zeros(TotD-1,TotD+1)];
    AiMa = AiMa - [zeros(TotD-1,2*TotD+1), eye(TotD-1), zeros(TotD-1,3*TotD+1), -eye(TotD-1), zeros(TotD-1,TotD)];
        
    % Append and delete the constraints
    Ai = [Ai; AiMc; AiMn; AiMa];
    Bi = [Bi; zeros(3*(TotD-1),1)];
    
    clear AiMc AiMn AiMa;
end

% Check if we've imposed positivity in components
if (SubPos == 1)&&(SubMon == 0)
    % Set up positivity constraint for compliers
    AiPc = [zeros(TotD,5*TotD), eye(TotD), zeros(TotD,2*TotD)];
    
    % Set up positivity constraint for never takers
    AiPn = [zeros(TotD,3*TotD) -eye(TotD), zeros(TotD,3*TotD), eye(TotD)];
    
    % Set up positivity constraint for always takers
    AiPa = [zeros(TotD,2*TotD) eye(TotD), zeros(TotD,3*TotD), -eye(TotD), zeros(TotD,TotD)];
    
    % Append and delete the constraints
    Ai = [Ai; AiPc; AiPn; AiPa];
    Bi = [Bi; zeros(3*TotD,1)];
    
    clear AiPc AiPn AiPa;    

elseif (SubPos == 1)&&(SubMon == 1)
    % Set up positivity constraint for compliers
    AiPc = [zeros(1,5*TotD + TotD-1), 1, zeros(1,2*TotD)];
    
    % Set up positivity constraint for never takers
    AiPn = [zeros(1,3*TotD + TotD-1), -1, zeros(1,3*TotD + TotD-1), 1];
    
    % Set up positivity constraint for always takers
    AiPa = [zeros(1,2*TotD + TotD-1), 1, zeros(1,3*TotD + TotD-1), -1, zeros(1,TotD)];
    
    % Append and delete the constraints
    Ai = [Ai; AiPc; AiPn; AiPa];
    Bi = [Bi; zeros(3,1)];
    
    clear AiPc AiPn AiPa;    
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Next start by computing the unconstrained estimator by 2SLS
Vt = NaN(momdim,1);         % Pre-allocate the target (YZ in (YZ - X'\beta)Z)
Vp = NaN(momdim,betadim);   % Pre-allocate the cov (ZX' in (YZ - X'\beta)Z))

% Fill these guys in - note the last parameter is the ATEs
for i = 1:momdim
    Vt(i) = mean(AllY(:,IndY(i)).*AllZ(:,IndZ(i)));       
    Vp(i,:) = mean(repmat(AllZ(:,IndZ(i)),1,betadim).*AllX(:,IndX(i,:)));
end

% Get the unconstrained minimizer (without partially ID ATE)
ShortVp = Vp(:,1:6*TotD);
ShortbetaU = (ShortVp'*ShortVp)\(ShortVp'*Vt);

% Compute the variance of the unconstrained estimator
V2SLS_err = NaN(N,momdim);     % Pre-allocate where we'll save residuals
for i = 1:momdim
    RelX = AllX(:,IndX(i,:));
    V2SLS_err(:,i) = (AllY(:,IndY(i))-RelX(:,1:6*TotD)*ShortbetaU).*AllZ(:,IndZ(i)) ;
end    

% Create the variance for 2SLS
V2SLS_meat = (V2SLS_err'*V2SLS_err)/N;
V2SLS = (ShortVp'*ShortVp)\((ShortVp'*V2SLS_meat*ShortVp)/(ShortVp'*ShortVp))/N;

% Create a weighting mtrix for inference
W0 = sqrtm(inv(V2SLS_meat));

clear V2SLS_meat ShortVp RelX ShortbetaU Vp Vt;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Turn to studentizing constraints and computing Rn. Note, we  studentize 
% only to help pick Rn, no real role otherwise. First we impute a variance 
% from id counterpart to LB and UB on Partial ID (e.g., for E[Y(1)|NT] we
% use the variance of E[Y(0)|NT] and similiarly for E[Y(0)|AT])
ImpB = [zeros(TotD,2*TotD), eye(TotD), zeros(TotD,3*TotD)];
ImpB = [ImpB; zeros(TotD,3*TotD), eye(TotD), zeros(TotD,2*TotD)];
ImpVarB = ImpB*V2SLS*ImpB';

% Stacking a copy because imputed variances for LB and UB are the same
ImpVarB = [diag(ImpVarB); diag(ImpVarB)];                               

% Next move on to computing the standardization for each row
AsyS = NaN(size(Bi));               
for i = 1:size(Ai,1)    
    % If the restrictions corresponds to an LB or UB used imputed
    if i <= 4*TotD
        AsyS(i) = sqrt(N)*sqrt(ImpVarB(i));
    end
    
    % If the restriction involves fully id parameters, just use those
    if i > 4*TotD
        AsyS(i) = sqrt(N)*sqrt(Ai(i,1:6*TotD)*V2SLS*Ai(i,1:6*TotD)');
    end    
end

% Now studentize Ai and Bi
AiS = Ai./repmat(AsyS,1, betadim);
BiS = Bi./AsyS;

% Select Rn to be used for monotonicity test (if not hard coded). Hard
% coding Rn is convenient when we want to set it to zero or infinity
if exist('pRn', 'var')
    TempBoot = NaN(B,1);                % Preallocate where we'll save draws
    
    % Substream at B + 1 because we'll use first B in bootstrap
    s = RandStream('CombRecursive', 'Seed', SEED);
    s.Substream = B + 1;  
    RandStream.setGlobalStream(s);

    % Again we'll impute lower and upper bound variances from corresponding
    % elements in the never taker always taker groups
    AuxImp = ImpB./repmat(AsyS(1:2*TotD),1,6*TotD);
    AuxImp = [AuxImp; AuxImp];
    
    if size(Ai,1) == 4*TotD
        ShortAiS = AuxImp;
    else
        ShortAiS = [AuxImp; AiS(4*TotD+1:end,1:6*TotD)];
    end
    
    % This piece of code just helps when V2SLS is large
    TempInd = sum(abs(ShortAiS))~= 0;
    
    for b = 1:B           
        TempBoot(b) = max(ShortAiS(:,TempInd)*sqrtm(V2SLS(TempInd',TempInd))*randn(sum(TempInd),1));            
    end

    Rn = max(quantile(TempBoot, pRn),0);
    
    clear TempBoot TempInd ShortAiS AuxImp;
end

clear ImpB ImpVarB;

% Finally call the test to get a pvalue for this null hypothesis
[SPval, betaR] = TestPoint(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, AiS, BiS, [], [], Rn, B, SEED);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Move on to getting point estimates of upper and lower bounds. 
% Pre-allocate variables and compute estimates
EstATE = NaN(size(BigATE,1),2);
ArgOptL = NaN(betadim,size(BigATE,1));
ArgOptU = NaN(betadim,size(BigATE,1));

for i = 1:size(EstATE,1)   
    % Set up the particulars of this optimization problem
    % Get the conditional probabilities for this group
    CondP = mean(XEdu).*BigATE(i,:);
    CondP = CondP'/sum(CondP);    
        
    % The vector f computes the ATE for those parameters
    f = zeros(betadim,1);    
    f(2*TotD+1:3*TotD) = CondP.*betaR(1:TotD);
    f(3*TotD+1:4*TotD) = -CondP.*betaR(TotD+1:2*TotD);
    f(5*TotD+1:6*TotD) = CondP.*(1-betaR(1:TotD)-betaR(TotD+1:2*TotD));
    f(6*TotD+1:7*TotD) = -CondP.*betaR(1:TotD);
    f(7*TotD+1:8*TotD) = CondP.*betaR(TotD+1:2*TotD);
    
    % Constraint observable variables to match betaR
    EstAe = [eye(6*TotD), zeros(6*TotD,2*TotD)];
    EstBe = betaR(1:6*TotD);    
    
    % Compute and store both optimal and maximizer
    optLB = linprog(f,Ai,Bi,EstAe,EstBe,[],[],[],linoptions);
    optUB = linprog(-f,Ai,Bi,EstAe,EstBe,[],[],[],linoptions);
    EstATE(i,:) = [f'*optLB, f'*optUB];
    ArgOptL(:,i) = optLB;
    ArgOptU(:,i) = optUB;
end

clear EstAe EstBe f optLB optUB CondP;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Turn to creating a CI for the ATE. 

if SPval >= (1 - CoverCI)
    % PreAllocate where we will save information
    CIAggATE = NaN(size(BigATE,1),2);
    CIpval = NaN(size(BigATE,1),2);          % Pvalue at the endpoint
    CIflagR = NaN(size(BigATE,1),2);         % The flags for the full stat at CI ends
    CIflagP = NaN(size(BigATE,1),2);         % Percentage success bootstrap at CI ends
    
    for i = 1:size(CIAggATE,1)
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Get a heuristic estimate of a standard deviation based on the ATE
        % decomposition to guide our grid search (but not inference)
        CondP = mean(XEdu).*BigATE(i,:);
        CondP = CondP'/sum(CondP);    
        
        % The vector IndV computes the part of the ATE that is ID
        IndV = zeros(6*TotD,1);
        IndV(2*TotD+1:3*TotD) = CondP.*betaR(1:TotD);
        IndV(3*TotD+1:4*TotD) = -CondP.*betaR(TotD+1:2*TotD);
        IndV(5*TotD+1:6*TotD) = CondP.*(1-betaR(1:TotD)-betaR(TotD+1:2*TotD));        
        RelS = sqrt(IndV'*V2SLS*IndV);
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Move to getting the lower bound of the CI for this variable
        % Create the weighting matrix for this problem - We'll use the
        % restricted estimators for the endpoints for the ATE moment        
        ATEL = f_GetATE(ArgOptL(:,i));
        W0 = [V2SLS_err, XEdu*((BigATE(i,:))'.*(ATEL-EstATE(i,1)))];
        W0 = sqrtm(inv(W0'*W0/N));        
                
        clear ATEL CondP;
         
        % Use this linear functional to define the function we need to zero
        findp = @(x) TestPointAggATE(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, AiS, BiS, [], [], BigATE(i,:), x, Rn, B, SEED)-(1-CoverCI);
    
        % A rough guess of whether we should reject
        Tstrt = EstATE(i,1) - 2*RelS;
        Tpval = TestPointAggATE(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, AiS, BiS, [], [], BigATE(i,:), Tstrt, Rn, B, SEED);
                    
        % A break to avoid getting stuck in loop
        sf = 1;
        
        if Tpval < (1-CoverCI)
            % If we already found a point we'd reject, set bounds
            x0 = [Tstrt; EstATE(i,1)];
            
        else            
            % If we are not rejecting, then move up until we reject
            while Tpval >= (1-CoverCI)
                % Move up by 0.2 standard deviations and try again
                Tstrt = Tstrt - max(0.2*RelS,0.01);
                Tpval = TestPointAggATE(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, AiS, BiS, [], [], BigATE(i,:), Tstrt, Rn, B, SEED);
                
                % A counter to make sure we are not stuck inside while loop
                sf = sf + 1;
                if sf == 50
                    break
                end
            end
                            
            % Once we've found the desired upper bound, store it
            x0 = [Tstrt; EstATE(i,1)];
        end
            
        % Do a grid search if we succesfully found the endpoints
        if sf < 50
            CIAggATE(i,1) = fzero(findp, x0, zeroptions);
            [CIpval(i,1),~,~,CIflagR(i,1),CIflagP(i,1)] = TestPointAggATE(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, AiS, BiS, [], [], BigATE(i,:), CIAggATE(i,1), Rn, B, SEED);
        end
             
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Move on to computing the upper bound of the CI for this variable                
        % Create the weighting matrix for this problem - We'll use the
        % restricted estimators for the endpoints for the ATE moment
        ATEU = f_GetATE(ArgOptU(:,i));
        W0 = [V2SLS_err, XEdu*((BigATE(i,:))'.*(ATEU-EstATE(i,2)))];
        W0 = sqrtm(inv(W0'*W0/N));        
                
        clear ATEU;
                        
        % Use this linear functional to define the function we need to zero
        findp = @(x) TestPointAggATE(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, AiS, BiS, [], [], BigATE(i,:), x, Rn, B, SEED)-(1-CoverCI);
        
        % Start setting up the search for the upper and lower bounds
        if SubPos == 1
            Tstrt = min(EstATE(i,2)+2*RelS,0);            
        else
            Tstrt = EstATE(i,2)+2*RelS;
        end
        
        Tpval = TestPointAggATE(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, AiS, BiS, [], [], BigATE(i,:), Tstrt, Rn, B, SEED);
        
        % Set the endpoints within which fzero will search for crossing
        sf = 1;

        if Tpval < (1-CoverCI)
            % If we already found a point we'd reject, set bounds
            x0 = [EstATE(i,2); Tstrt];        
        else            
            if SubPos == 1
                % If we are not rejecting, then move up until we reject            
                while (Tpval >= (1-CoverCI))&&(Tstrt <0)
                    % Move up by 0.2 standard deviations and try again
                    Tstrt = min(Tstrt + max(0.2*RelS,0.01),0);
                    Tpval = TestPointAggATE(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, AiS, BiS, [], [], BigATE(i,:), Tstrt, Rn, B, SEED);
                
                    % A counter to make sure we are not stuck inside while loop
                    sf = sf + 1;
                    if sf == 50
                        break
                    end
                end                                
            else                
                % If we are not rejecting, then move up until we reject            
                while Tpval >= (1-CoverCI)
                    % Move up by 0.2 standard deviations and try again
                    Tstrt = Tstrt + max(0.2*RelS,0.01);
                    Tpval = TestPointAggATE(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, AiS, BiS, [], [], BigATE(i,:), Tstrt, Rn, B, SEED);
                
                    % A counter to make sure we are not stuck inside while loop
                    sf = sf + 1;
                    if sf == 50
                        break
                    end
                end
            end
                                
            % Once we've found the desired upper bound, store it
            x0 = [EstATE(i,2); Tstrt];
        end
                    
        % Do a grid search if we succesfully found the endpoints
        if (sf < 50)&&(SubPos == 0)
            CIAggATE(i,2) = fzero(findp, x0, zeroptions);
            [CIpval(i,2), ~, ~, CIflagR(i,2), CIflagP(i,2)] = TestPointAggATE(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, AiS, BiS, [], [], BigATE(i,:), CIAggATE(i,2), Rn, B, SEED);                        
            
        elseif (sf < 50)&&(SubPos == 1)&&(Tpval < (1-CoverCI))
             CIAggATE(i,2) = fzero(findp, x0, zeroptions);
             [CIpval(i,2), ~, ~, CIflagR(i,2), CIflagP(i,2)] = TestPointAggATE(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, AiS, BiS, [], [], BigATE(i,:), CIAggATE(i,2), Rn, B, SEED);            
             
        elseif (sf < 50)&&(SubPos == 1)&&(Tpval >= (1-CoverCI))&&(Tstrt == 0)
            CIAggATE(i,2) = 0;
            [CIpval(i,2), ~, ~, CIflagR(i,2), CIflagP(i,2)] = TestPointAggATE(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, AiS, BiS, [], [], BigATE(i,:), CIAggATE(i,2), Rn, B, SEED);            
        end                
    end        
end   


% Clear variables we don't want to save, and save the output
clear AllX AllY AllZ ArgOptL ArgOptU AsyS betadim findp i IndV IndX IndY IndZ linoptions momdim RelS sf TotD Tpval Tstrt V2SLS V2SLS_err W0 x0 XEdu zeroptions AiS BiS;

if or(Rn == 0, Rn == Inf)
    save(['ATEAgg_SubMon' num2str(SubMon) '_SubPos' num2str(SubPos) '_Rn' num2str(Rn) '.mat'])
else
    save(['ATEAgg_SubMon' num2str(SubMon) '_SubPos' num2str(SubPos) '_pRn' num2str(pRn) '.mat'])
end   


