%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code is part of the empirical application based on Angrist and Evans
% 1998. The file below allows us to test for whether the individual LATEs
% satisfy certain shape restrictions, and obtain confidence regions for the
% individual LATEs while imposing certain shape restrictions across them.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set up some basic variables/parameters for the model
SEED = 210;                                 % Set the seed for the random number generator
B = 500;                                    % The number of bootstrap iterations                     
CoverCI = 0.95;                             % Coverage CI
pRn = 0.95;                                 % The quantile of sup norm Rn is chosen from
%Rn = 0;                                    % Hard code Rn if we want to set it + infty or zero
NonPos = 0;                                 % An indicator for whether we have a negativity constraint in place

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set up preferences for fzero, while will be used to find ends of CIs
zeroptions = optimset('Display','iter'); 
zeroptions.TolX = 1e-5;
zeroptions.TolFun = 1/B;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load the data and keep some of the basic information we want
load ProcAE;

N = size(AllVars,1);           % The sample size
D = AllVars.morekids;          % Treatment: Indicator for more than 2 kids
Y = AllVars.workedm;           % Dependent: Indicator for whether mother worked
Z = AllVars.samesex;           % Instrument: Sex composition of first two kids
edu = AllVars.educm;           % Mother's education

clear AllVars;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Next prepare the variables for our inference procedures
XEdu = [(edu < 9) (edu == 9) (edu == 10) (edu == 11) (edu == 12) (edu == 13) (edu== 14) (edu == 15) (edu == 16) (edu > 16)];
AllY = Y;                                             % All Ys in moments
AllX = [XEdu XEdu.*repmat(D,1,size(XEdu,2))];         % All Xs in moments
AllZ = [XEdu XEdu.*repmat(Z,1,size(XEdu,2))];         % All Zs in moments

% Declare some helpful dimensions we will use throughout
TotD = size(XEdu,2);            % Number of education dummies
betadim = 2*TotD;               % Number of parameters
momdim = 2*TotD;                % Number of moments

% Put together indicators for what variables enter in each moment. NOTE:
% For this module this is not entirely necessarily, but it becomes helpful
% in other parts of the code where different Xs, Zs, Ys appear in different
% moments of our estimation/inference procedure
IndY = ones(betadim,1);                 % Use the same Y in all mometns
IndX = repmat(1:betadim,momdim,1);      % Use all Xs in each moment
IndZ = (1:momdim)';                     % Use all instruments
 
clear D Y Z edu XEdu;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set up the matrix of shape constraints on the LATEs. Default is monotonic
Ai = [eye(TotD-1) zeros(TotD-1,1)] + [zeros(TotD-1,1) -eye(TotD-1)];
Ai = [zeros(TotD-1,TotD) Ai];
Bi = zeros(TotD-1,1);                                      

% If in addition we want to impose non-posi tivity, then append extra constr
if NonPos == 1                                          
    Ai = [Ai; zeros(1,2*TotD-1), 1];
    Bi = [Bi; 0];                     
end

% Set up a matrix indicating what coefficeints we want CIs for
BigAe = [zeros(TotD) eye(TotD)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Next start by computing the unconstrained estimators for the LATEs. NOTE:
% The code below is more general than we need for this application because
% in this problem the same Xs and Ys are appearing in all the moments
Vt = NaN(momdim,1);         % Pre-allocate the target (i.e. YZ in (YZ - X'\beta)Z)
Vp = NaN(momdim,betadim);   % Pre-allocate the cov (i.e. ZX' in (YZ - X'\beta)Z))

for i = 1:momdim
    Vt(i) = mean(AllY(:,IndY(i)).*AllZ(:,IndZ(i)));       
    Vp(i,:) = mean(repmat(AllZ(:,IndZ(i)),1,betadim).*AllX(:,IndX(i,:)));
end

% Compute the unconstrained estimator for all the parameters
betaU = (Vp'*Vp)\(Vp'*Vt);

% Compute the variance/covariance matrix of the unconstrained estimator
V2SLS_err = NaN(N,momdim);     % Pre-allocate where we'll save residuals
for i = 1:momdim    
    V2SLS_err(:,i) = (AllY(:,IndY(i))-AllX(:,IndX(i,:))*betaU).*AllZ(:,IndZ(i)) ;
end    

V2SLS_meat = (V2SLS_err'*V2SLS_err)/N;
V2SLS = (Vp'*Vp)\((Vp'*V2SLS_meat*Vp)/(Vp'*Vp))/N;

% Create a weighting matrix for inference when we apply our statistic
W0 = sqrtm(inv(V2SLS_meat));

clear V2SLS_meat V2SLS_err;

% Compute the CI based on the normal approximation for unconstrained LATEs
LateU = BigAe*betaU;
VLateU = BigAe*V2SLS*BigAe';
Ncrit = norminv(0.5+CoverCI/2);

CILateU = [LateU - Ncrit*diag(VLateU.^(1/2)), LateU + Ncrit*diag(VLateU.^(1/2))];

clear Ncrit;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Turn to testing whether the imposed shape restriction is reasonable
% Begin by studentizing the inequality restrictions
AiS = Ai./(sqrt(N)*repmat(sqrt(diag(Ai*V2SLS*Ai')),1,betadim));
BiS = Bi./(sqrt(N)*sqrt(diag(Ai*V2SLS*Ai')));

% Select Rn to be used for our test (if not hard coded). 
if exist('pRn', 'var')
    TempBoot = NaN(B,1);               % Preallocate where we'll save draws
    
    % Substream at B + 1 because we'll use first B in bootstrap
    s = RandStream('CombRecursive', 'Seed', SEED);
    s.Substream = B + 1;  
    RandStream.setGlobalStream(s);

    % Extract the entries we need for monotonicty constraint - this avoids
    % computing the square root of too large a matrix in simulations
    TempInd = sum(abs(Ai))~= 0;
    
    for b = 1:B           
        TempBoot(b) = max(AiS(:,TempInd)*sqrtm(V2SLS(TempInd',TempInd))*randn(sum(TempInd),1));            
    end

    Rn = max(quantile(TempBoot, pRn),0);
    
    clear TempBoot TempInd;
end

% Finally call the test to get a pvalue for this null hypothesis
[SPval, betaR, IValR, SPflagR, SPflagP] = TestPoint(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, AiS, BiS, [], [], Rn, B, SEED);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Turn to building confidence regions through test inversion. Only proceed
% if we didn't reject the validity of the inequality constraints.
if SPval >= (1 - CoverCI)
    
    % PreAllocate where we will save information
    CILateR = NaN(size(BigAe,1),2);         % The endpoints of the CI
    CIpval = NaN(size(BigAe,1),2);          % Pvalue at the endpoint
    CIflagR = NaN(size(BigAe,1),2);         % The flags for the full stat at CI ends
    CIflagP = NaN(size(BigAe,1),2);         % Percentage success bootstrap at CI ends
    
    % We'll work ourselves through the linear functionals we want
    for i = 1:size(BigAe,1)
        % Back out the linear functional of interest
        Ae = BigAe(i,:);

        % Now that we are imposing an equality restriction, we recompute Rn
        % Get the variance of the restricted estimator
        PR = eye(betadim) - ((Vp'*W0*Vp)\(Ae'*Ae))/(Ae*((Vp'*W0*Vp)\Ae'));        
        RV2SLS = PR*V2SLS*PR';
        
        % Studentize and drop constraints with zero variance (which
        % correspond to constraints whose value is implied by = constraints.
        AuxS = sqrt(N)*sqrt(diag(Ai*RV2SLS*Ai'));
        AiS = Ai./(repmat(AuxS,1,betadim));
        BiS = Bi./AuxS;        
        AiS(AuxS==0,:) = [];
        BiS(AuxS==0,:) = [];
        
        clear AuxS;
        
        % Select Rn to be used for our test (if not hard coded). 
        if exist('pRn', 'var')
            TempBoot = NaN(B,1);               % Preallocate where we'll save draws
    
            % Substream at B + 1 because we'll use first B in bootstrap
            s = RandStream('CombRecursive', 'Seed', SEED);
            s.Substream = B + 1;  
            RandStream.setGlobalStream(s);

            % Extract the entries we need for monotonicty constraint - this avoids
            % computing the square root of too large a matrix in simulations
            TempInd = sum(abs(Ai))~= 0;
    
            for b = 1:B
                % NOTE: We don't take sqrt of RV2SLS directly because it is
                % rank defficient. Instead we factor out PR which is idemp
                TempBoot(b) = max(AiS(:,TempInd)*PR(TempInd',TempInd)*sqrtm(V2SLS(TempInd',TempInd))*randn(sum(TempInd),1));                                                       
            end

            Rn = max(quantile(TempBoot, pRn),0);
    
            clear TempBoot TempInd;
        end
        
        clear PR RV2SLS;
        
        % Use this linear functional to define the function we need to zero        
        findp = @(x) TestPoint(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, AiS, BiS, Ae, x, Rn, B, SEED)-(1-CoverCI);
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Start with the lower bound. To help find the endpoint of the CI
        % it is helpful to give fzero a point at which we reject.
        Tstrt = CILateU(i,1);
        Tpval = TestPoint(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, AiS, BiS, Ae, Tstrt, Rn, B, SEED);
        
        % Set the endpoints within which fzero will search for crossing
        sf = 1;
        
        if Tpval < (1-CoverCI)
            % If we already found a point we'd reject, set bounds for fzero
            x0 = [Ae*betaR; Tstrt];
        
        else            
            % If we are not rejecting, then move up until we find a point
            while Tpval >= (1-CoverCI)
                % Move up by 0.2 standard deviations and try again
                Tstrt = Tstrt - 0.2*sqrt(VLateU(i,i));
                Tpval = TestPoint(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, AiS, BiS, Ae, Tstrt, Rn, B, SEED);
                
                % A counter to make sure we are not stuck inside while loop
                sf = sf + 1;
                if sf == 50
                    break
                end
            end
                            
            % Once we've found the desired upper bound, store it
            x0 = [Ae*betaR; Tstrt];
        end
            
        % Do a grid search if we succesfully found the endpoints
        if sf < 50
            CILateR(i,1) = fzero(findp, x0, zeroptions);
            [CIpval(i,1),~,~, CIflagR(i,1), CIflagP(i,1)] = TestPoint(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, AiS, BiS, Ae, CILateR(i,1), Rn, B, SEED);
        end
                      
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Move to the upper bound of the CI. Same as the lower bound, but
        % now need to account for the logical bound of zero if pos constr.        
        if NonPos == 1
            Tstrt = min(CILateU(i,2),0);
        else
            Tstrt = CILateU(i,2);
        end
        Tpval = TestPoint(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, AiS, BiS, Ae, Tstrt, Rn, B, SEED);
        
        % Set the endpoints within which fzero will search for crossing
        sf = 1;
        
        if Tpval < (1-CoverCI)
            % If we already found a point we'd reject, set bounds
            x0 = [Ae*betaR; Tstrt];        
        else            
            if NonPos == 1
                % If we are not rejecting, then move up until we reject            
                while (Tpval >= (1-CoverCI))&&(Tstrt <0)
                    % Move up by 0.2 standard deviations and try again
                    Tstrt = min(Tstrt + 0.2*sqrt(VLateU(i,i)),0);
                    Tpval = TestPoint(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, AiS, BiS, Ae, Tstrt, Rn, B, SEED);
                
                    % A counter to make sure we are not stuck inside while loop
                    sf = sf + 1;
                    if sf == 50
                        break
                    end
                end                                
            else                
                % If we are not rejecting, then move up until we reject            
                while Tpval >= (1-CoverCI)
                    % Move up by 0.2 standard deviations and try again
                    Tstrt = Tstrt + 0.2*sqrt(VLateU(i,i));
                    Tpval = TestPoint(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, AiS, BiS, Ae, Tstrt, Rn, B, SEED);
                
                    % A counter to make sure we are not stuck inside while loop
                    sf = sf + 1;
                    if sf == 50
                        break
                    end
                end
            end
                                
            % Once we've found the desired upper bound, store it
            x0 = [Ae*betaR; Tstrt];
        end
            
        % Do a grid search if we succesfully found the endpoints
        if (sf < 50)&&(NonPos == 0)
            CILateR(i,2) = fzero(findp, x0, zeroptions);
            [CIpval(i,2),~,~, CIflagR(i,2), CIflagP(i,2)] = TestPoint(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, AiS, BiS, Ae, CILateR(i,2), Rn, B, SEED);
            
        elseif (sf < 50)&&(NonPos == 1)&&(Tpval < (1-CoverCI))
             CILateR(i,2) = fzero(findp, x0, zeroptions);
             [CIpval(i,2),~,~, CIflagR(i,2), CIflagP(i,2)] = TestPoint(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, AiS, BiS, Ae, CILateR(i,2), Rn, B, SEED);
             
        elseif (sf < 50)&&(NonPos == 1)&&(Tpval >= (1-CoverCI))&&(Tstrt == 0)
            CILateR(i,2) = 0;
            [CIpval(i,2),~,~, CIflagR(i,2), CIflagP(i,2)] = TestPoint(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, AiS, BiS, Ae, CILateR(i,2), Rn, B, SEED);            
        end        
    end
end

% Clear variables we don't want to save, and save the output
clear AllX AllY AllZ b betadim findp i IndY IndX IndZ momdim s SEED sf Tpval Tstrt x0 zeroptions W0 RV2SLS PR AiS BiS Vt Vp;

if or(Rn == 0, Rn == Inf)
    save(['LateInd_NonPos' num2str(NonPos) '_Rn' num2str(Rn) '.mat'])
else
    save(['LateInd_NonPos' num2str(NonPos) '_pRn' num2str(pRn) '.mat'])
end    