function [pval, betaR, IValR, flagR, flagP] = TestPoint(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, Ai, Bi, Ae, Be, Rn, B, SEED)
%TestPoint This fuction implements the test in Chernozhukov, Newey, and
% Santos for linear moments with the structure E[(Y-Xbeta)Z] = 0. The inputs
% AllY, AllX, AllZ contain all the Ys, Xs, and Zs. The IndY, IndX, and IndZ
% indicate what Ys, Xs, Zs appear in each moment to allow for different
% conditioning variables in each modemnt. 
   
% Set up some parameters we'll need for the procedure
quadoptions = optimoptions('quadprog');
quadoptions.Algorithm = 'interior-point-convex';
quadoptions.Display = 'off';
quadoptions.TolFun = 1e-10;

% Read some basics from the data
N = size(AllY,1);                       % Sample size backed of the Y                          
momdim = size(W0,1);                    % The number of moments (off weighting matrix)
betadim = size(IndX,2);                   % Number of parameters (off constraints)

% Square W0 for convenience (this is how it enters in many subproblems 
W0s = W0'*W0;

% Next get the variables we need to build test statistic
Vt = NaN(momdim,1);         % Pre-allocate the target (i.e. YZ in (YZ - X'\beta)Z)
Vp = NaN(momdim,betadim);   % Pre-allocate the cov (i.e. ZX' in (YZ - X'\beta)Z))

% Fill these guys in using the indicators
for i = 1:momdim
    Vt(i) = mean(AllY(:,IndY(i)).*AllZ(:,IndZ(i)));
    Vp(i,:) = mean(repmat(AllZ(:,IndZ(i)),1,betadim).*AllX(:,IndX(i,:)));
end

% Next obtain the shape constrained estimator for beta
H = N*Vp'*W0s*Vp; H = (H+H')/2;
f = -N*Vp'*W0s*Vt;
[betaR, ~, flagR] = quadprog(H,f,Ai,Bi,Ae,Be,[],[],[],quadoptions);

% Use the constrained estimator to compute restricted minimum
IValR = sqrt(N)*norm(W0*(Vt - Vp*betaR));

% If test statistic is (numerically) zero, then we are done
if IValR <= sqrt(quadoptions.TolFun)   
    pval = 1;
    flagP = 1;
else
    % Get the residuals before bootstrapping the objective function
    BootRes = NaN(N,momdim);        % Pre-allocate where we'll save residuals
    for i = 1:momdim
        BootRes(:,i) = (AllY(:,IndY(i)) - AllX(:,IndX(i,:))*betaR).*AllZ(:,IndZ(i));
    end
    BootRes = BootRes - repmat(mean(BootRes),N,1);
                                       
    % Create the derivative we will use in optimization problem
    BootDer = - Vp;
        
    % Create bootstrap level constraints
    BootAe = Ae;
    BootBe = Be*0;
    
    % Create the bootstrap inequality constraints
    if ~isempty(Ai)
        % Set constraints only if we have inequalities, otherwise pass
        BootAi = Ai;         
        BootBi = sqrt(N)*max(0,-(Rn + Ai*betaR-Bi));
    else
        BootAi = Ai;
        BootBi = Bi;
    end        
    
    % Create an array where we will save the bootstrap values
    BootStatR = NaN(1,B);       % Will Store Restricted minimum   
    BootFlag = NaN(1,B);        % Will Flag something went wrong

    % Finally, we need to move into loops to compute the pvalues for the   
    parfor b = 1:B               
        % Set the random number generator for replicability              
        s = RandStream('CombRecursive', 'Seed', SEED);
        s.Substream = b;  
        RandStream.setGlobalStream(s);
        
        % Get the bootstrap quantity for this bootstrap
        What = (N^(1/2))*mean(BootRes.*repmat(randn(N,1),1,momdim))';                                
                        
        % Next compute the inputs into the quadratic programming problem        
        H = BootDer'*W0s*BootDer; H = (H+H')/2;
        f = BootDer'*W0s*What;                                  
               
        % Call the quadratic program for the boostrap constrained programme.
        [BootBetaR, ~, flagB] = quadprog(H,f,BootAi, BootBi,BootAe,BootBe,[],[],zeros(betadim,1),quadoptions);
        BootStatR(b) = norm(W0*(What + BootDer*BootBetaR),2);
        BootFlag(b) = flagB;                       
    end   
    
    % Compute the p-value, and FlagP to get a sense of whether bootstrap
    % optimization problems behaved correctly. NOTE: We take all draws
    % regardless noting that if the bootstrap did not converge properly,
    % then it leads to less powerful inference, but does control size.
    pval = mean(BootStatR > repmat(IValR,1,B));
    flagP = mean(BootFlag==1);

end

end

