function [pval, betaR, IValR, flagR, flagP] = TestPointAggATE(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, Ai, Bi, Ae, Be, AggInd, null, Rn, B, SEED)
%TestPointAggATE This fuction implements the test in Chernozhukov, Newey
% and Santos for conducting inference on an aggregate ATE. The only
% difference with the TestPoint function, is that the current file appends
% one last (nonlinear) moment for the aggregate ATE restriction.
   
% Set up some parameters we'll need for the procedure
% quadoptions are for the quadratic problem used in the bootstrap
quadoptions = optimoptions('quadprog');
quadoptions.Algorithm = 'interior-point-convex';
quadoptions.Display = 'off';
quadoptions.TolFun = 1e-10;

% foptions are for the non-convex criterion used for the full statistic
foptions = optimoptions('fmincon');
foptions.Display = 'off';
foptions.Algorithm = "interior-point";
foptions.SpecifyConstraintGradient = true;
foptions.SpecifyObjectiveGradient = true;
foptions.TolFun = 1e-10;

% The number of starting points for multisearch when computing test stat
TotStart = 50;

% Read some basics from the data
N = size(AllY,1);                       % Sample size                           
momdim = size(W0,1)-1;                  % The number of moments (off weighting matrix)
betadim  = size(IndX,2);                % The dimension of beta (sans ATE)
TotD = length(AggInd);                  % The number of education dummies

% A useful variable having the probabilities of each relevant edu cell
agg = mean(AllX(:,1:TotD)).*AggInd;

% Square W0 since that's how it enters in some problems
W0s = W0'*W0;

% Next get the variables we need to build test statistic
Vt = NaN(momdim,1);         % Pre-allocate the target (i.e. YZ in (YZ - X'\beta)Z)
Vp = NaN(momdim,betadim);   % Pre-allocate the cov (i.e. ZX' in (YZ - X'\beta)Z))

% Fill these guys in moment by moment
for i = 1:momdim
    Vt(i) = mean(AllY(:,IndY(i)).*AllZ(:,IndZ(i)));
    Vp(i,:) = mean(repmat(AllZ(:,IndZ(i)),1,betadim).*AllX(:,IndX(i,:)));
end

% Move on to get an initial guess for this problem - a couple of steps
% First: find minimizer subject to linear ineq and eq constraitns
H = N*Vp'*W0s(1:end-1,1:end-1)*Vp; H = (H+H')/2;
f = -N*Vp'*W0s(1:end-1,1:end-1)*Vt;
PreX0 = quadprog(H,f,Ai,Bi,Ae,Be,[],[],[],quadoptions);

% Second: Get as close as possible to the hypothesized ATE by matching PreX0
% on all the identified components -- i.e. only change partial ID aspects
f = zeros(betadim,1); 
f(2*TotD+1:3*TotD) = agg'.*PreX0(1:TotD);
f(3*TotD+1:4*TotD) = -agg'.*PreX0(TotD+1:2*TotD);
f(5*TotD+1:6*TotD) = agg'.*(1-PreX0(1:TotD)-PreX0(TotD+1:2*TotD));
f(6*TotD+1:7*TotD) = -agg'.*PreX0(1:TotD);
f(7*TotD+1:8*TotD) = agg'.*PreX0(TotD+1:2*TotD);
H = f*f'; H = (H+H')/2;
f = -f*null*sum(agg);
TAe = [Ae; eye(6*TotD), zeros(6*TotD,2*TotD)];
TBe = [Be; PreX0(1:6*TotD)];
PreX1 = quadprog(H,f,Ai,Bi,TAe,TBe,[],[],[],quadoptions);

clear H f PreX0 TAe TBe;

% Third: Compute the ATE corresponding to PreX1 and use it to create new linear
% ATE moment which we will use to compute final starting point
[PreATE, dPreATE] = f_GetATE(PreX1);
gr = agg*dPreATE;
AugVp = [Vp; gr];
H = N*AugVp'*W0s*AugVp; H = (H+H')/2;
AugVt = [Vt; (sum(agg)*null -agg*PreATE + gr*PreX1)];
f = -N*AugVp'*W0s*AugVt;
X0 = quadprog(H,f,Ai,Bi,Ae,Be,[],[],[],quadoptions);

clear PreATE PreATEa PreATEn PreATEc gr AugVP AugVt PreX1;

% Create the objective and constraint functions
fobj = @(x) f_AggATEobj(x, Vt, Vp, sqrt(N)*W0, agg, null);

% We also set up wide logical lower and upper bounds on the parameters
% (they follow from Y \in [0,1]). They don't have information in a
% statistical sense but help discipline the optimizer from veering off
LB = [zeros(2*TotD,1); -ones(4*TotD,1); zeros(2*TotD,1)];
UB = [ones(4*TotD,1); 2*ones(TotD,1); ones(3*TotD,1)];

% Set up the optimization problem. We'll run multistart as a check
FullProb = createOptimProblem('fmincon','objective', fobj, 'x0', X0, 'Aineq', Ai, 'bineq', Bi, 'Aeq', Ae, 'beq', Be, 'lb', LB, 'ub', UB, 'nonlcon', [], 'options', foptions);
MS = MultiStart('UseParallel', true, 'Display', 'off');

% Set the random number generator used by MS for replicability 
s = RandStream('CombRecursive', 'Seed', SEED);
s.Substream = 2*B+1;  
RandStream.setGlobalStream(s);

% Call multisearch and save output
[betaR, ~, flagR] = run(MS,FullProb,TotStart);

% Finally save the test statistic
IValR = sqrt(fobj(betaR));

% If test statistic is zero, then we are done
if IValR <= sqrt(foptions.OptimalityTolerance)
    pval = 1;    
else
    % Get the residuals before bootstrapping the objective function   
    BootRes = NaN(N,momdim+1);        % Pre-allocate where we'll save residuals
    for i = 1:momdim
        BootRes(:,i) = (AllY(:,IndY(i)) - AllX(:,IndX(i,:))*betaR).*AllZ(:,IndZ(i));        
    end
    
    % Fill in the last moment (nonlinear one leading to ATE)
    [ATER, dATER] = f_GetATE(betaR);    
    BootRes(:,end) = (AllX(:,1:TotD).*repmat(AggInd,N,1))*(ATER - null);    
    
    % And recenter to get the final residual
    BootRes = BootRes - repmat(mean(BootRes),N,1);           
    
    % Create the derivative we will use in optimization problem
    BootDer = [-Vp; agg*dATER];
                   
    % Create bootstrap level constraints
    BootAe = Ae;
    BootBe = Be*0;
    
    % Create the bootstrap inequality constraints
    if ~isempty(Ai)
        % Set constraints only if we have inequalities, otherwise pass
        BootAi = Ai;         
        BootBi = sqrt(N)*max(0,-(Rn + Ai*betaR-Bi));
    else
        BootAi = Ai;
        BootBi = Bi;
    end        
    
    % Create an array where we will save the bootstrap values
    BootStatR = NaN(1,B);       % Will Store Restricted minimum       
    
    % Finally, we need to move into loops to compute the pvalues for the   
    parfor b = 1:B               
        % Set the random number generator for replicability              
        s = RandStream('CombRecursive', 'Seed', SEED);
        s.Substream = b;  
        RandStream.setGlobalStream(s);
        
        % Get the bootstrap quantity for this bootstrap        
        What = (N^(1/2))*mean(BootRes.*repmat(randn(N,1),1,momdim+1))';                                
        
        % Next compute the inputs into the quadratic programming problem        
        H = BootDer'*W0s*BootDer; H = (H+H')/2;
        f = BootDer'*W0s*What;  
                
        % Call the quadratic program for the boostrap constrained programme.
        [BootBetaR, ~, flagB] = quadprog(H,f,BootAi, BootBi,BootAe,BootBe,[],[],zeros(betadim,1),quadoptions);
        BootStatR(b) = norm(W0*(What + BootDer*BootBetaR),2);
        BootFlag(b) = flagB;
    end       
    
    % Compute the p-value, and FlagP to get a sense of whether bootstrap
    % optimization problems behaved correctly. NOTE: We take all draws
    % regardless noting that if the bootstrap did not converge properly,
    % then it leads to less powerful inference, but does control size.
    pval = mean(BootStatR > repmat(IValR,1,B));
    flagP = mean(BootFlag==1);
        
end

end
