function [pval, betaR, IValR, flagR, flagP] = TestPointIndATE(AllY, AllX, AllZ, IndY, IndX, IndZ, W0, Ai, Bi, Ae, Be, pos, null, Rn, B, SEED)
% TestPointIndATE This fuction implements the CNS test on the AE dataset for
%   conducting inference on the ATE parameter under different restrictions
%   on the treatment effects for compliers, never takers and always takers
   
% Options for the quadratic problems used to start optimizers
quadoptions = optimoptions('quadprog');
quadoptions.Algorithm = 'interior-point-convex';
quadoptions.Display = 'off';
quadoptions.TolFun = 1e-10;

% foptions are for the non-linear constraint in the optimization
foptions = optimoptions('fmincon');
foptions.Algorithm = 'sqp';
foptions.Display = 'off';
foptions.SpecifyConstraintGradient = true;
foptions.SpecifyObjectiveGradient = true;

% The number of starting points for multisearch when computing test stat
TotStart = 50;

% Read some basics from the data
N = size(AllY,1);                       % Sample size                           
momdim = size(W0,1);                    % The number of moments (off weighting matrix)
betadim = size(IndX,2);                 % Number of parameters (off constraints)
TotD = betadim/8;

% Square W0 since that's how it enters in some problems
W0s = W0'*W0;

% Next get the variables we need to build test statistic
Vt = NaN(momdim,1);         % Pre-allocate target (YZ in (YZ - X'\beta)Z)
Vp = NaN(momdim,betadim);   % Pre-allocatee cov (ZX' in (YZ - X'\beta)Z))

% Fill these guys in moment by moment
for i = 1:momdim
    Vt(i) = mean(AllY(:,IndY(i)).*AllZ(:,IndZ(i)));
    Vp(i,:) = mean(repmat(AllZ(:,IndZ(i)),1,betadim).*AllX(:,IndX(i,:)));
end

% Create the objective and constraint functions for the test statistic
ft = sqrt(N)*(W0*Vt); fV = sqrt(N)*(W0*Vp);
fobj = @(x) f_ATEobj(x,ft,fV);
fcon = @(x) f_ATEcon(x,pos,null);

% Move on to get an initial guess for this problem - a couple of steps
% First: find minimizer subject to linear ineq and eq constraitns
H = Vp'*W0s*Vp; H = (H+H')/2;
f = -Vp'*W0s*Vt;
PreX0 = quadprog(H,f,Ai,Bi,Ae,Be,[],[],[],quadoptions);

% Second: Get as close as possible to the hypothesized ATE by matching PreX0
% on all the identified components -- i.e. only change partial ID aspects
f = zeros(8*TotD,1); 
f(2*TotD+pos) = PreX0(pos);    
f(3*TotD+pos) = -PreX0(TotD+pos);      
f(5*TotD+pos) = 1-PreX0(pos)-PreX0(TotD+pos);        
f(6*TotD+pos) = -PreX0(pos);
f(7*TotD+pos) = PreX0(TotD+pos);
H = f*f'; H = (H+H')/2;
f = -f*null;
TAe = [Ae; eye(6*TotD), zeros(6*TotD,2*TotD)];
TBe = [Be; PreX0(1:6*TotD)];
PreX1 = quadprog(H,f,Ai,Bi,TAe,TBe,[],[],[],quadoptions);

% Third: Solve the quadratic problem linearizing the constraints
H = Vp'*W0s*Vp; H = (H+H')/2;
f = -Vp'*W0s*(Vt - Vp*PreX1);
TAi = Ai;
TBi = Bi - Ai*PreX1;
[~, TBe, ~, TAe] = f_ATEcon(PreX1, pos, null);
TAe = [TAe'; Ae];
TBe = [-TBe; Be];
X0 = quadprog(H,f,TAi,TBi,TAe,TBe,[],[],[],quadoptions);
X0 = X0 + PreX1;

clear H f PreX0 PreX1 TAe TBe TAi TBi;

% We also set up wide logical lower and upper bounds on the parameters
% (they follow from Y \in [0,1]). They don't have information in a
% statistical sense but help discipline the optimizer from veering off
LB = [zeros(2*TotD,1); -ones(4*TotD,1); zeros(2*TotD,1)];
UB = [ones(4*TotD,1); 2*ones(TotD,1); ones(3*TotD,1)];

% Set up the optimization problem. We'll run multistart as a check
FullProb = createOptimProblem('fmincon','objective', fobj, 'x0', X0, 'Aineq', Ai, 'bineq', Bi, 'Aeq', Ae, 'beq', Be, 'lb', LB, 'ub', UB, 'nonlcon', fcon, 'options', foptions);
MS = MultiStart('UseParallel', true, 'Display', 'off');

% Set the random number generator used by MS for replicability 
s = RandStream('CombRecursive', 'Seed', SEED);
s.Substream = 2*B+1;  
RandStream.setGlobalStream(s);

% Call multisearch and save output
[betaR, ~, flagR] = run(MS,FullProb,TotStart);

% Finally save the test statistic
IValR = sqrt(fobj(betaR));

% If test statistic is zero, then we are done
if IValR <= sqrt(foptions.TolFun)
    pval = 1;
    flagP = 1;
else
    % Get the residuals before bootstrapping the objective function   
    BootRes = NaN(N,momdim);
    for i = 1:momdim
        BootRes(:,i) = (AllY(:,IndY(i)) - AllX(:,IndX(i,:))*betaR).*AllZ(:,IndZ(i));        
    end
    BootRes = BootRes - repmat(mean(BootRes),N,1);
        
    % Create bootstrap level constraints
    BootAe = Ae;
    BootBe = Be*0;
    
    % Create the bootstrap inequality constraints
    if ~isempty(Ai)
        % Set constraints only if we have inequalities, otherwise pass
        BootAi = Ai;         
        BootBi = sqrt(N)*max(0,-(Rn + Ai*betaR-Bi));
    else
        BootAi = Ai;
        BootBi = Bi;
    end        
    
    % Adjust the nonlinear equality constraint
    fcon = @(x) f_BootATEcon(x,pos,null,betaR,N);
    
    % Create an array where we will save the bootstrap values
    BootStatR = NaN(1,B);       % Will Store Restricted minimum       
    BootFlag = NaN(1,B);        % Will Flag something went wrong    
               
    % Adjust options to abort if we find a bootstrap value below the test
    % statistic, since that is all that matters for the pvalue
    foptions.ObjectiveLimit = 0;
    
    % Finally, we need to move into loops to compute the pvalues for the   
    parfor b = 1:B               
        % Set the random number generator for replicability              
        s = RandStream('CombRecursive', 'Seed', SEED);
        s.Substream = b;  
        RandStream.setGlobalStream(s);
        
        % Get the bootstrap quantity for this bootstrap        
        What = (N^(1/2))*mean(BootRes.*repmat(randn(N,1),1,momdim))';                                
        
        % Create the objective and constraint functions
        ft = W0*What; fV = W0*Vp;
        fobj = @(x) f_BootATEobj(x,ft,fV,IValR);                                     
        
        % Get a starting point for bootstrap problem by solving the quadratic 
        % program we get by replacing ATE constraint with zero derivative
        % of ATE from betaR in direction of h
        H = Vp'*W0s*Vp; H = (H+H')/2;
        f = -Vp'*W0s*What;
        [~, ~, ~, Gceq] = f_ATEcon(betaR, pos, null);                    
        X0 = quadprog(H,f,BootAi,BootBi,[BootAe; Gceq'],[BootBe;0],[],[],zeros(betadim,1),quadoptions);        
        
        % Finally, call fmincon to compute the true bootstrap statistic. We'll do two runs        
        [~, v1, flag1] = fmincon(fobj, X0, BootAi, BootBi, BootAe, BootBe, [], [], fcon, foptions);         
        
        % If we did not beat IValR we'll try a second starting point just
        % in case we hit a local minimum in the first run
        if sqrt(v1+IValR^2) >= IValR
            [~, v2, flag2] = fmincon(fobj, 0*X0, BootAi, BootBi, BootAe, BootBe, [], [], fcon, foptions);
            
            % In case all went wrong evaluate function at 0
            v3 = fobj(X0*0);        
            BootStatR(b) = min([sqrt(v1+IValR^2),sqrt(v2+IValR^2),sqrt(v3+IValR^2)]);    
            BootFlag(b) = (max(flag1,flag2) > 0)||(BootStatR(b) < IValR);
        else
            % If we already found a succesful point, then move on
            BootStatR(b) = sqrt(v1+IValR^2);
            BootFlag(b) = 1;
        end                          
    end
                                                
    % As a final check, see if we can improve on the failed optimizations
    for b = 1:B
        % Only rerun problem if we've failed to converge
        if BootFlag(b) == 0
            % Set the random number generator for replicability              
            s = RandStream('CombRecursive', 'Seed', SEED);
            s.Substream = b;  
            RandStream.setGlobalStream(s);
        
            % Get the bootstrap quantity for this bootstrap        
            What = (N^(1/2))*mean(BootRes.*repmat(randn(N,1),1,momdim))';                                
        
            % Create the objective and constraint functions
            ft = W0*What; fV = W0*Vp;
            fobj = @(x) f_BootATEobj(x,ft,fV,IValR);
            X0 = zeros(betadim,1);
            
            % Set up the optimization problem. We'll run multistart as a check
            FullProb = createOptimProblem('fmincon','objective', fobj, 'x0', X0, 'Aineq', BootAi, 'bineq', BootBi, 'Aeq', BootAe, 'beq', BootBe, 'lb', [], 'ub', [], 'nonlcon', fcon, 'options', foptions);
            MS = MultiStart('UseParallel', true, 'Display', 'iter');

            % Call multisearch to try to get a convergence point
            [~, v4, Bootflag] = run(MS,FullProb,TotStart);
            v4 = v4+IValR^2;
                        
            BootFlag(b) = (Bootflag>0)||(Bootflag == -3);
            BootStatR(b) = min(sqrt(v4), BootStatR(b));
                
        end
    end     
    
        
    % Compute the p-value, and FlagP to get a sense of whether bootstrap
    % optimization problems behaved correctly. NOTE: We take all draws
    % regardless noting that if the bootstrap did not converge properly,
    % then it leads to less powerful inference, but does control size.
    pval = mean(BootStatR > repmat(IValR,1,B));
    flagP = mean(BootFlag>0);
        
end

end